package com.ccwangluo.ccim;

import android.Manifest;
import android.media.MediaPlayer;

import androidx.fragment.app.FragmentActivity;

import com.ccwangluo.ccim.listener.OnConnectListener;
import com.ccwangluo.ccim.listener.ResultCallback;
import com.ccwangluo.ccim.listener.ResultEventCallback;
import com.ccwangluo.ccim.manage.CallBackManager;
import com.ccwangluo.ccim.manage.SocketManageer;
import com.ccwangluo.ccim.util.MediaHelper;
import com.google.protobuf.RepeatedFieldBuilder;
import com.permissionx.guolindev.PermissionX;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import ccim.protocol.Channel;
import ccim.protocol.Chat;
import ccim.protocol.Message;
import ccim.protocol.MessageSpec;
import ccim.protocol.Push;

public class CCIMClient {

    private final Logger logger = Logger.getLogger(CCIMClient.class.getName());

    private SocketManageer socketManageer;
    private CallBackManager<Chat.ChatResponseMessage> callBackManager;

    private CCIMClient() {
    }

    private static CCIMClient mInstance;

    public static CCIMClient getInstance() {
        if (mInstance == null) {
            synchronized (CCIMClient.class) {
                if (mInstance == null) {
                    mInstance = new CCIMClient();
                }
            }
        }
        return mInstance;
    }

    public void initSDK(String appKey, String userName,String extData, Long userId, Long ts, String sign, OnConnectListener resultCallback) {
        callBackManager = new CallBackManager<>();
        socketManageer = new SocketManageer(callBackManager);
        socketManageer.initChat(appKey, userName,extData, userId,ts,sign,resultCallback);
    }


    public void registerMessageListener(ResultEventCallback<Chat.ChatResponseMessage> resultEventCallback) {
        callBackManager.addCallBack(0, chatResponseMessage -> {
            logger.info("registerMessageListener : " + chatResponseMessage);
            if (chatResponseMessage.hasPushPrivateMsgRes()) {
                MessageSpec.ChatMessage pushPrivateMsgRes = chatResponseMessage.getPushPrivateMsgRes().getMessage();
                pushPrivateMessageAck(pushPrivateMsgRes.getSenderUserId(), pushPrivateMsgRes.getMsgId());
            }
            if (chatResponseMessage.hasPushChannelMsgRes()) {
                Push.PushChannelMessageResponse channelMsgRes = chatResponseMessage.getPushChannelMsgRes();
                pushChannelMessage(channelMsgRes.getChannelId(), channelMsgRes.getMessage().getMsgId());
            }
            resultEventCallback.onSuccess(chatResponseMessage);
        });
    }


    /**
     * 获取联系人列表
     */
    public void getContactList(ResultEventCallback<Chat.PullContactsResponse> resultEventCallback) {
        Chat.PullContactsRequest builder = Chat.PullContactsRequest.newBuilder()
                .build();
        int id = socketManageer.sendRequest(builder);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasPullContactsRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    Chat.PullContactsResponse pullContactsRes = chatResponseMessage.getPullContactsRes();
                    resultEventCallback.onSuccess(pullContactsRes);
                }
            }

            @Override
            public void onFailed(int errorCode) {
                resultEventCallback.onFailed(errorCode);
            }
        });

    }


    /**
     * 同步离线消息
     */
    public void getOfflineMessage(Long lastPullMsgId,ResultEventCallback<List<Message.PullOfflineMessageResponse.OfflineMessage>> resultEventCallback) {
        Message.PullOfflineMessageRequest offlineMessageRequest = Message.PullOfflineMessageRequest.newBuilder()
                .setLastPulledMsgId(lastPullMsgId)
                .build();
        int id = socketManageer.sendRequest(offlineMessageRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasPullOfflineMsgRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    List<Message.PullOfflineMessageResponse.OfflineMessage> offlineMsgsList = chatResponseMessage.getPullOfflineMsgRes().getOfflineMsgsList();
                    resultEventCallback.onSuccess(offlineMsgsList);
                }
            }

            @Override
            public void onFailed(int errorCode) {
                resultEventCallback.onFailed(errorCode);
            }
        });

    }

    /**
     * 搜索好友
     */
    public void searchUser(String userName, ResultEventCallback<List<Chat.SearchResponse.User>> resultEventCallback) {
        Chat.SearchRequest searchRequest = Chat.SearchRequest.newBuilder().setUsername(userName).build();
        int id = socketManageer.sendRequest(searchRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasSearchRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    List<Chat.SearchResponse.User> searchRes = chatResponseMessage.getSearchRes().getUsersList();
                    resultEventCallback.onSuccess(searchRes);
                }
            }

            @Override
            public void onFailed(int errorCode) {
                resultEventCallback.onFailed(errorCode);
            }
        });
    }

    /**
     * 添加好友
     */
    public void addContact(Long userId, ResultEventCallback<Chat.AddContactResponse.AddResult> callback) {
        Chat.AddContactRequest addContactRequest = Chat.AddContactRequest.newBuilder().setUserId(userId).build();
        int id = socketManageer.sendRequest(addContactRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasAddContactRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    Chat.AddContactResponse addContactRes = chatResponseMessage.getAddContactRes();
                    Chat.AddContactResponse.AddResult result = addContactRes.getResult();
                    callback.onSuccess(result);
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });

    }

    /**
     * 联系人申请列表
     */
    public void getContactReqList(ResultEventCallback<Chat.ContactRequestsResponse> callback) {
        Chat.ContactRequestsRequest contactRequestsRequest = Chat.ContactRequestsRequest.newBuilder().build();
        int id = socketManageer.sendRequest(contactRequestsRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasContactRequestsRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    Chat.ContactRequestsResponse contactRequestsRes = chatResponseMessage.getContactRequestsRes();
                    callback.onSuccess(contactRequestsRes);
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });

    }

    /**
     * 验证联系人申请
     */
    public void checkContactReq(Long reqUserId, boolean accept, ResultCallback callback) {
        Chat.CheckContactReqRequest checkContactReqRequest = Chat.CheckContactReqRequest.newBuilder()
                .setReqUserId(reqUserId)
                .setAccept(accept)
                .build();
        int id = socketManageer.sendRequest(checkContactReqRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasCheckContactReqRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    callback.onSuccess();
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }

    /**
     * 创建联系人群组
     */
    public void createContactGroup(String name, ResultEventCallback<Chat.CreateContactGroupResponse> callback) {
        Chat.CreateContactGroupRequest createContactGroupRequest = Chat.CreateContactGroupRequest.newBuilder()
                .setName(name)
                .build();
        int id = socketManageer.sendRequest(createContactGroupRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasCreateContactGroupRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    Chat.CreateContactGroupResponse createContactGroupRes = chatResponseMessage.getCreateContactGroupRes();
                    callback.onSuccess(createContactGroupRes);
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }


    /**
     * 移动联系人所在的群组
     */
    public void moveContactGroup(Long groupId, Long userId, ResultCallback callback) {
        Chat.MoveContactGroupRequest moveContactGroupRequest = Chat.MoveContactGroupRequest.newBuilder()
                .setContactGroupId(groupId)
                .setContactUserId(userId)
                .build();
        int id = socketManageer.sendRequest(moveContactGroupRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasMoveContactGroupRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    callback.onSuccess();
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }

    /**
     * 删除联系人
     */
    public void deleteContact(Long userId, ResultCallback callback) {
        Chat.DeleteContactRequest deleteContactRequest = Chat.DeleteContactRequest.newBuilder()
                .setContactUserId(userId)
                .build();
        int id = socketManageer.sendRequest(deleteContactRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasDeleteContactRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    callback.onSuccess();
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }

    /**
     * 拉黑联系人
     */
    public void blockContact(Long userId, Boolean mute, ResultCallback callback) {
        Chat.BlockContactRequest blockContactRequest = Chat.BlockContactRequest.newBuilder()
                .setContactUserId(userId)
                .setMute(mute)
                .build();
        int id = socketManageer.sendRequest(blockContactRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasBlockContactRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    callback.onSuccess();
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }

    /**
     * 举报违规用户
     */
    public void reportIllegalUser(String title, String content, Long userId, ResultCallback callback) {
        Chat.ReportIllegalUserRequest reportIllegalUserRequest = Chat.ReportIllegalUserRequest.newBuilder()
                .setContent(content)
                .setIllegalUserId(userId)
                .setTitle(title)
                .build();
        int id = socketManageer.sendRequest(reportIllegalUserRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasReportIllegalUserRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    callback.onSuccess();
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }

    /**
     * 发送消息
     */
    public void sendTextMessage(String content, Long reciveUserId, ResultCallback callback) {
        Message.SendMessageRequest messageRequest = Message.SendMessageRequest.newBuilder()
                .setContent(content)
                .setMsgType(MessageSpec.MessageType.MESSAGE_TYPE_TEXT)
                .setReceiverUserId(reciveUserId).build();
        int id = socketManageer.sendRequest(messageRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasSendMsgRes()) {
                    callback.onSuccess();
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }


    /**
     * 创建频道
     */
    public void createChannel(String channelName,String tag,boolean isOpen,List<Long> userIds, ResultEventCallback<Long> callback) {
        Channel.CreateChannelRequest.Builder builder = Channel.CreateChannelRequest.newBuilder()
                .setName(channelName)
                .setTag(tag)
                .setOpen(isOpen);
        for (Long id : userIds) {
            builder.addUserIds(id);
        }
        int id = socketManageer.sendRequest(builder.build());
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasCreateChannelRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    Channel.CreateChannelResponse createChannelRes = chatResponseMessage.getCreateChannelRes();
                    callback.onSuccess(createChannelRes.getChannelId());
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }

    /**
     * 加入频道
     */
    public void joinChannel(Long channelId, ResultCallback callback) {
        Channel.JoinChannelRequest.Builder builder = Channel.JoinChannelRequest.newBuilder()
                .setChannelId(channelId);
        int id = socketManageer.sendRequest(builder.build());
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasJoinChannelRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    callback.onSuccess();
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }

    /**
     * 移除频道成员
     */
    public void removeChannelMember(Long channelId,Long userId, ResultCallback callback) {
        Channel.RemoveMemberRequest.Builder builder = Channel.RemoveMemberRequest.newBuilder()
                .setChannelId(channelId)
                .setUserId(userId);
        int id = socketManageer.sendRequest(builder.build());
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasRemoveMemberRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    callback.onSuccess();
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }

    /**
     * 退出频道
     */
    public void exitChannel(Long channelId, ResultCallback callback) {
        Channel.ExitChannelRequest.Builder builder = Channel.ExitChannelRequest.newBuilder()
                .setChannelId(channelId);
        int id = socketManageer.sendRequest(builder.build());
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasRemoveMemberRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    callback.onSuccess();
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }


    /**
     * 发送频道消息
     */
    public void sendChannelMessage(String content, Long channelId, ResultCallback callback) {
        Message.SendChannelMessageRequest sendChannelMessageRequest = Message.SendChannelMessageRequest.newBuilder()
                .setContent(content)
                .setMsgType(MessageSpec.MessageType.MESSAGE_TYPE_TEXT)
                .setChannelId(channelId)
                .build();
        int id = socketManageer.sendRequest(sendChannelMessageRequest);
        callBackManager.addCallBack(id, new ResultEventCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasSendChannelMsgRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                    callback.onSuccess();
                }
            }

            @Override
            public void onFailed(int errorCode) {
                callback.onFailed(errorCode);
            }
        });
    }

    /**
     * 推送私人消息成功ack
     */
    public void pushPrivateMessageAck(Long contactUserId, Long msgId) {
        Push.PushPrivateMessageAckRequest pushPrivateMessageAckRequest = Push.PushPrivateMessageAckRequest.newBuilder()
                .setContactUserId(contactUserId)
                .setMsgId(msgId)
                .build();
        int id = socketManageer.sendRequest(pushPrivateMessageAckRequest);
        callBackManager.addCallBack(id, chatResponseMessage -> logger.info(String.valueOf(chatResponseMessage)));
    }


    /**
     * 推送渠道消息成功ack
     */
    public void pushChannelMessage(Long channelId, Long msgId) {
        Push.PushChannelMessageAckRequest channelMessageAckRequest = Push.PushChannelMessageAckRequest.newBuilder()
                .setChannelId(channelId)
                .setMsgId(msgId)
                .build();
        int id = socketManageer.sendRequest(channelMessageAckRequest);
        callBackManager.addCallBack(id, chatResponseMessage -> logger.info(String.valueOf(chatResponseMessage)));
    }

    /**
     * 开始录音
     */

    public void startRecordAudioMessage(FragmentActivity activity) {
        PermissionX.init(activity)
                .permissions(Manifest.permission.RECORD_AUDIO)
                .onExplainRequestReason((scope, deniedList, beforeRequest) -> scope.showRequestReasonDialog(deniedList, "即将申请的权限是程序必须依赖的权限", "我已明白"))
                .onForwardToSettings((scope, deniedList) -> scope.showForwardToSettingsDialog(deniedList, "您需要去应用程序设置当中手动开启权限", "我已明白"))
                .request((allGranted, grantedList, deniedList) -> {
                    if (allGranted) {
                        MediaHelper.getInstance().prepare(activity);
                    }
                });
    }

    public void stopAndSendAudioMessage(){
        MediaHelper.getInstance().release();
        String filePath = MediaHelper.getInstance().getFilePath();

    }

    public void cancleAudioMessage(){
        MediaHelper.getInstance().cancel();
    }


    public void startPlayAudio(String filePath){
        MediaHelper.getInstance().play(filePath);
    }

}
